/*
 * WorkFlowXMLDoc.java
 *
 * Created on 15 de octubre de 2007, 16:10
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
/* -------------------------------------------------------------------------- */
/* Copyright 2007-2008, Iván Fernández Hernández <rayban1984@gmail.com>       */
/*                                                                            */
/* Licensed under the Apache License, Version 2.0 (the "License"); you may    */
/* not use this file except in compliance with the License. You may obtain    */
/* a copy of the License at                                                   */
/*                                                                            */
/* http://www.apache.org/licenses/LICENSE-2.0                                 */
/*                                                                            */
/* Unless required by applicable law or agreed to in writing, software        */
/* distributed under the License is distributed on an "AS IS" BASIS,          */
/* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   */
/* See the License for the specific language governing permissions and        */
/* limitations under the License.                                             */
/* -------------------------------------------------------------------------- */
package paqInterfaz;


import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.JOptionPane;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Text;
import org.jdom.input.SAXBuilder;

import org.jdom.output.XMLOutputter;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import pSintactico.ErrorSintacticoException;
import pSintactico.ErroresSintacticosDialog;
import pSintactico.SintacticoRanking;
import pSintactico.SintacticoRequisitos;


/**
 *
 * @author globus
 */
public class WorkFlowXMLDoc {
    
    
    public static int SINGLE = 0;
    
    public static int MULTIPLE = 1;
    
    public static int MPI = 2;
    /** Creates a new instance of WorkFlowXMLDoc */
    
    GUI interfaz;
    File file = null;
    Document document;
    
    public WorkFlowXMLDoc(GUI interfaz,String fileName) {
        
        this.interfaz = interfaz;
        
        file = new File(fileName);
        
        this.crearDocumento();
        
        
    }
    
    private Fichero leerEjecutable(Element ejecutable) throws URISyntaxException {
        Fichero executable = null;
        boolean local = false;
        if (ejecutable.getName().compareTo("Executable") != 0)
            return null;
        
        if (ejecutable.getAttribute("local").getValue().compareTo("yes") == 0)
            local = true;
        else
            local = false;
        
        if (ejecutable.getContent().size() > 0) {
            String fichero = ((Text)ejecutable.getContent().get(0)).getTextNormalize();
            
            executable = Fichero.constructorFichero(true,true,fichero,local);
            return executable;
        }
        
        return null;
    }
    
    private String leerArgumentos(Element arguments) throws ErrorSintacticoException {
        String argumentos = "";
        
        List lista = arguments.getChildren();
        
        for (int i = 0;i<lista.size();i++) {
            Element argument = (Element)lista.get(i);
            
            if (argument.getName().compareTo("Argument") == 0) {
                argumentos += ((Text)argument.getContent().get(0)).getTextNormalize()+ " ";
            }
            
            else
                throw new ErrorSintacticoException(0,"Unexpected tag found!");
            
        }
        
        return argumentos;
        
        
    }
    
    private String leerVariables(Element environment) throws ErrorSintacticoException {
        String variables = "";
        
        List lista = environment.getChildren();
        
        for (int i = 0;i<lista.size();i++) {
            Element variable = (Element) lista.get(i);
            if (variable.getName().compareTo("Variable") == 0) {
                variables += ((Text)variable.getContent().get(0)).getTextNormalize()+ " ";
            } else
                throw new ErrorSintacticoException(0,"Unexpected tag found!");
        }
        
        return variables;
    }
    
    
    private int leerTipo(Element tipo) {
        if (tipo.getName().compareTo("Type") == 0) {
            if (((Text)tipo.getContent().get(0)).getTextNormalize().compareTo("SINGLE") == 0)
                return this.SINGLE;
            else if (((Text)tipo.getContent().get(0)).getTextNormalize().compareTo("MULTIPLE") == 0)
                return this.MULTIPLE;
            else if (((Text)tipo.getContent().get(0)).getTextNormalize().compareTo("MPI") == 0)
                return this.MPI;
        }
        
        return -1;
    }
    
    private int leerNP(Element tipo) throws ErrorSintacticoException {
        if (tipo.getName().compareTo("Type") == 0) {
            if (tipo.getAttribute("NP").getValue() !=  null) {
                if (tipo.getAttribute("NP").getValue().compareTo("") != 0)
                    return Integer.parseInt(tipo.getAttribute("NP").getValue());
            }
            
            throw new ErrorSintacticoException(0,"NP tag not found!");
            
            
        }
        
        else
            throw new ErrorSintacticoException(0,"Type Tag not found!");
        
        
    }
    
    
    private ArrayList leerFicherosEntrada(Element InFiles) throws URISyntaxException, ErrorSintacticoException {
        ArrayList ficheros_entrada = new ArrayList();
        
        if (InFiles.getName().compareTo("INPUT_FILES") == 0) {
            List listaficheros = InFiles.getChildren();
            
            for (int i = 0;i<listaficheros.size();i++) {
                Element file = (Element)listaficheros.get(i);
                
                if (file.getName().compareTo("FILE") == 0) {
                    boolean local = false;
                    if (file.getAttribute("local").getValue().compareTo("yes") == 0)
                        local = true;
                    else
                        local = false;
         
                    Fichero aux = Fichero.constructorFichero(true,false,((Text)file.getContent().get(0)).getTextNormalize(),local);
                    
                    ficheros_entrada.add(aux);
                } else throw new ErrorSintacticoException(0,"Unexpected tag found while looking for FILE tag");
            }
        }
        
        return ficheros_entrada;
    }
    
    private ArrayList leerFicherosSalida(Element OutFiles) throws URISyntaxException, ErrorSintacticoException {
        
        ArrayList ficheros_salida = new ArrayList();
        
        if (OutFiles.getName().compareTo("OUTPUT_FILES") == 0) {
            List listaficheros = OutFiles.getChildren();
            
            for (int i = 0;i<listaficheros.size();i++) {
                
                Element file = (Element)listaficheros.get(i);
                
                if (file.getName().compareTo("FILE") == 0) {
                    boolean local = false;
                    if (file.getAttribute("local").getValue().compareTo("yes") == 0)
                        local = true;
                    else
                        local = false;
                    
                    Fichero aux = Fichero.constructorFichero(true,false,((Text)file.getContent().get(0)).getTextNormalize(),local);
                    
                    ficheros_salida.add(aux);
                    
                } else throw new ErrorSintacticoException(0,"Unexpected tag found while looking for FILE tag");
            }
        }
        
        return ficheros_salida;
    }
    
    
    private Fichero leerSTDIN(Element entrada) throws URISyntaxException, ErrorSintacticoException {
        Fichero fichero = new Fichero();
        
        if (entrada.getName().compareTo("Streams") == 0) {
            List lista = entrada.getChildren();
            
            for (int i = 0;i<lista.size();i++) {
                Element aux = (Element)lista.get(i);
                
                if (aux.getName().compareTo("STDIN") == 0) {
                    boolean local = false;
                    
                    if (aux.getAttribute("local").getValue().compareTo("yes") == 0)
                        local = true;
                    else
                        local = false;
                    
                    fichero = Fichero.constructorFichero(true,false,((Text)aux.getContent().get(0)).getTextNormalize(),local);
                    return fichero;
                }
            }
        }
        
        else throw new ErrorSintacticoException(0,"Unexpected tag found while looking for STDIN tag");
        
        return fichero;
    }
    
    private Fichero leerSTDERR(Element stderr) throws URISyntaxException, ErrorSintacticoException {
        
        Fichero fichero = new Fichero();
        
        if (stderr.getName().compareTo("Streams") == 0) {
            List lista = stderr.getChildren();
            
            for (int i = 0;i<lista.size();i++) {
                Element aux = (Element)lista.get(i);
                
                if (aux.getName().compareTo("STDERR") == 0) {
                    boolean local = false;
                    if (aux.getAttribute("local").getValue().compareTo("local") == 0)
                        local = true;
                    else
                        local = false;
                    fichero = Fichero.constructorFichero(true,false,((Text)aux.getContent().get(0)).getTextNormalize(),local);
                    return fichero;
                }
            }
        }
        
        else throw new ErrorSintacticoException(0,"Unexpected tag found while looking for STDERR tag");
        
        return fichero;
    }
    
    private Fichero leerSTDOUT(Element salida) throws URISyntaxException, ErrorSintacticoException {
        Fichero fichero = new Fichero();
        
        if (salida.getName().compareTo("Streams") == 0) {
            List lista = salida.getChildren();
            
            for (int i = 0;i<lista.size();i++) {
                Element aux = (Element)lista.get(i);
                
                if (aux.getName().compareTo("STDOUT") == 0) {
                    boolean local = false;
                    if (aux.getAttribute("local").getValue().compareTo("local") == 0)
                        local = true;
                    else
                        local = false;
                    fichero = Fichero.constructorFichero(true,false,((Text)aux.getContent().get(0)).getTextNormalize(),local);
                    return fichero;
                }
            }
        }
        
        else throw new ErrorSintacticoException(0,"Unexpected tag found while looking for STDOUT tag");
        
        return fichero;
    }
    
    private Tarea leerTarea(Element task) throws ErrorSintacticoException, URISyntaxException {
        
        
        Tarea tarea = null;
        String nombre = null;
        
        if (task.getAttribute("Name").getValue() != null && task.getAttribute("Name").getValue().compareTo("") != 0)
            nombre = task.getAttribute("Name").getValue();
        
        
        
        else
            throw new ErrorSintacticoException(0,"The task has no valid name, it's imposible to load it");
        
        if (task.getChildren().size() > 0)
        {
            Element bulk= (Element) task.getChildren().get(0);
            
            if (bulk.getName().compareTo("BulkJobInfo") == 0)
            {
                tarea = new TareaMultiple();
          
                
                try {
                if (bulk.getAttribute("number_of_tasks").getValue() != null)
                    ((TareaMultiple)tarea).numeroTareas = Integer.parseInt(bulk.getAttribute("number_of_tasks").getValue());
                else
                    ((TareaMultiple)tarea).numeroTareas = 1;
                }catch(NumberFormatException e)
                {
                    throw new ErrorSintacticoException(0,"The number of tasks of a job array must be a number!");
                }
            }
            else 
                tarea = new Tarea();
                
        }
        
        else throw new ErrorSintacticoException(0,"Important information of the task could not be found!");
        
        
              
        tarea.nombre = nombre;
//        if (tarea.nombre.contains(" ") == true)
  //          throw new ErrorSintacticoException(0, "One of the tasks contains spaces in their names");
        
        List componentes = task.getChildren();
        
        for (int i = 0;i<componentes.size();i++) {
            
            Element elemento = (Element)componentes.get(i);
            
            if (elemento.getName().compareTo("Executable") == 0)
                tarea.ejecutable = this.leerEjecutable(elemento);
            
            else if (elemento.getName().compareTo("Arguments") == 0) {
                tarea.argumentos = this.leerArgumentos(elemento);
            }
            
            else if (elemento.getName().compareTo("Environment") == 0)
                tarea.variables = this.leerVariables(elemento);
            
            else if (elemento.getName().compareTo("Type") == 0) {
                tarea.tipo = this.leerTipo(elemento);
                
                tarea.tipo = this.leerNP(elemento);
            }
            
            else if (elemento.getName().compareTo("Scheduling") == 0) {

                
                List listasched = elemento.getChildren();
                
                for (int j = 0;j<listasched.size();j++) {
                    Element what = (Element)listasched.get(j);
                    
                    if (what.getName().compareTo("Interval") == 0)
                        tarea.intervaloPlanificacion = Integer.parseInt(((Text)what.getContent().get(0)).getTextNormalize());
                    
                    else if (what.getName().compareTo("Reschedule_threshold") == 0)
                        tarea.umbralPlanificacion = Integer.parseInt(((Text)what.getContent().get(0)).getTextNormalize());
                    
                    else if (what.getName().compareTo("Deadline") == 0)
                    {
                        try {
                        if (what.getContent().size() > 0)
                            
                            tarea.tiempoDeadline = new Date(Long.parseLong(((Text)what.getContent().get(0)).getTextNormalize()));
                        else
                            tarea.tiempoDeadline = new Date();
                        }catch(Exception e)
                        {
                            throw new ErrorSintacticoException(0,"The date format is invalid!");
                        }
                    }
                    
                }
                
                
            }
            
            else if (elemento.getName().compareTo("INPUT_FILES") == 0) {
                tarea.ficherosEntrada = this.leerFicherosEntrada(elemento);
            }
            
            else if (elemento.getName().compareTo("OUTPUT_FILES") == 0) {
                tarea.ficherosSalida = this.leerFicherosSalida(elemento);
            }
            
            else if (elemento.getName().compareTo("Streams") == 0) {
                tarea.stdin = this.leerSTDIN(elemento);
                
                tarea.stdout = this.leerSTDOUT(elemento);
                
                tarea.stderr = this.leerSTDERR(elemento);
                
            }
            
            else if (elemento.getName().compareTo("CHECKPOINTING") == 0) {
                //restartfile interval url
                
                List lista_check = elemento.getChildren();
                
                for (int j = 0;i<lista_check.size();j++) {
                    Element elemento_check = (Element) lista_check.get(j);
                    
                    if (elemento_check.getName().compareTo("Interval") == 0)
                        tarea.intervaloCheckpoint = Integer.parseInt(((Text)elemento_check.getContent().get(0)).getTextNormalize());
                    
                    else if (elemento_check.getName().compareTo("RESTART_FILES") == 0) {
                        Element restart_file = (Element) elemento_check.getChildren().get(0);
                        if (restart_file.getName().compareTo("FILE") == 0)
                            tarea.restartFile = Fichero.constructorFicheroRestart(((Text)restart_file.getContent().get(0)).getTextNormalize(),false);
                        
                        
                    } else if (elemento_check.getName().compareTo("CHECKPOINT_URL") == 0) {
                        tarea.checkpointURL = ((Text)elemento_check.getContent().get(0)).getTextNormalize();
                    }
                    
                    else
                        throw new ErrorSintacticoException(0,"Unexpected tag!");
                    
                    
                }
            }
            
            else if (elemento.getName().compareTo("REQUIREMENTS") == 0) {
                
                String requisitos = "";
                if (elemento.getContent().size() > 0)
                    requisitos = ((Text)elemento.getContent().get(0)).getTextNormalize();
                
                tarea.requisitosTarea = requisitos;
                
                
            }
            
            else if (elemento.getName().compareTo("RANKING") == 0) {
                
                String ranking = "";
                if (elemento.getContent().size() > 0)
                    ranking = ((Text)elemento.getContent().get(0)).getTextNormalize();
                
                tarea.rankingTarea = ranking;
            }
            
            else if (elemento.getName().compareTo("PERFORMANCE_INFORMATION") == 0) {
                List performance = elemento.getChildren();
                
                for (int j = 0;j<performance.size();j++) {
                    Element perform = (Element) performance.get(j);
                    
                    if (perform.getName().compareTo("SUSPENSION_TIMEOUT") == 0)
                        tarea.tiempo_maximo_de_suspension = Integer.parseInt(((Text)perform.getContent().get(0)).getTextNormalize());
                    
                    else if (perform.getName().compareTo("CPULOAD_THRESHOLD") == 0)
                        tarea.umbral_carga_cpu = Integer.parseInt(((Text)perform.getContent().get(0)).getTextNormalize());
                    
                    else if (perform.getName().compareTo("MONITOR") == 0) {
                        boolean local = false;
                        if (perform.getAttribute("local") != null &&
                                perform.getAttribute("local").getValue().compareTo("yes") == 0)
                            local = true;
                        else
                            local = false;
                        if (perform.getContent().size() > 0)
                            tarea.programa_monitor = Fichero.constructorFichero(true,true,((Text)perform.getContent().get(0)).getTextNormalize(),
                                    local);
                        
                        else
                            tarea.programa_monitor = new Fichero();
                    } else
                        throw new ErrorSintacticoException(0,"Unexpected tag!");
                }
            }
            
            else if (elemento.getName().compareTo("FAULT_TOLERANCE") == 0) {
                List fault = elemento.getChildren();
                
                for (int j = 0;j< fault.size();j++) {
                    Element fault_element = (Element) fault.get(j);
                    
                    if (fault_element.getName().compareTo("Reschedule_on_failure") == 0) {
                        if (((Text)fault_element.getContent().get(0)).getTextNormalize().compareTo("yes") == 0)
                            tarea.planificarSiFallo = true;
                        else if (((Text)fault_element.getContent().get(0)).getTextNormalize().compareTo("no") == 0)
                            tarea.planificarSiFallo = false;
                        
                        else throw new ErrorSintacticoException(0,"It was expected yes or no, but they were not found!");
                        
                    }
                    
                    else if (fault_element.getName().compareTo("Retries") == 0)
                        tarea.numeroReintentos = Integer.parseInt(((Text)fault_element.getContent().get(0)).getTextNormalize());
                    
                    else
                        throw new ErrorSintacticoException(0,"Unexpected tag!");
                }
            }
            
            
            else if (elemento.getName().compareTo("WRAPPER") == 0) {
                boolean local = false;
                if (elemento.getAttribute("local").getValue().compareTo("yes") == 0)
                    local = true;
                else
                    local = false;
                
                
                tarea.wrapper = Fichero.constructorFichero(true,true,((Text)elemento.getContent().get(0)).getTextNormalize(),local);
            }
            
            else if (elemento.getName().compareTo("PRE_WRAPPER") == 0) {
                boolean local = false;
                if (elemento.getAttribute("local").getValue().compareTo("yes") == 0)
                    local = true;
                else
                    local = false;
                tarea.preWrapper = Fichero.constructorFichero(true,true,((Text)elemento.getContent().get(0)).getTextNormalize(),local);
            }
            
            else if (elemento.getName().compareTo("PRE_WRAPPER_ARGUMENTS") == 0) {
                tarea.argumentosPreWrapper = this.leerArgumentos(elemento);
            }
            
            else if (elemento.getName().compareTo("POSITION") == 0) {
                tarea.x_pos = Double.valueOf(elemento.getAttribute("x").getValue());
                tarea.y_pos = Double.valueOf(elemento.getAttribute("y").getValue());
            }
            
        }
        
        
        
        return tarea;
        
    }
    
    
    private Dependencia leerDependencia(Element Dependency,ArrayList tareas) throws ErrorSintacticoException, URISyntaxException {
        Dependencia dependencia = null;
        
        if (Dependency.getName().compareTo("Dependency") == 0) {
            String nombre = Dependency.getAttribute("Name").getValue();
            String tarea1=null, tarea2=null;
            ArrayList ficheros = new ArrayList();
            
            Tarea task1=null,task2 = null;
            
            if (nombre != null && nombre.compareTo("") != 0) {
                List partesdep = Dependency.getChildren();
                
                for (int i = 0;i<partesdep.size();i++) {
                    Element aux = (Element) partesdep.get(i);
                    
                    if (aux.getName().compareTo("Task1") == 0) {
                        
                        if (aux.getContent().size()  != 1)
                            throw new ErrorSintacticoException(0,"Tarea1 bad defined while analyzing dependency "+nombre);
                        
                        tarea1 = ((Text)aux.getContent().get(0)).getTextNormalize();
                        
                        
                        
                    }
                    
                    else if (aux.getName().compareTo("Task2") == 0) {
                        
                        if (aux.getContent().size() != 1)
                            throw new ErrorSintacticoException(0,"Tarea2 bad defined while analyzing dependency "+nombre);
                    
                        tarea2 = ((Text)aux.getContent().get(0)).getTextNormalize();
                        
                        
                    }
                    
                    else if (aux.getName().compareTo("Files") == 0) {
                        List lista_ficheros = aux.getChildren();
                        
                        for (int j = 0;j<lista_ficheros.size();j++) {
                            Element fichero = (Element) lista_ficheros.get(j);
                            
                            if (fichero.getName().compareTo("File") == 0) {
                                boolean local = false;
                                if (fichero.getAttribute("local") != null && fichero.getAttribute("local").getValue().compareTo("local") == 0)
                                    local = true;
                                else
                                    local = false;
                                
                                Fichero nuevo_fichero = Fichero.constructorFichero(false,false,
                                        ((Text)fichero.getContent().get(0)).getTextNormalize(),local);
                                ficheros.add(nuevo_fichero); //Se añade a la lista temporal de ficheros de la dependencia
                            }
                            
                            else
                                throw new ErrorSintacticoException(0,"Unexpected files found while analyzing dependency files: "+nombre);
                        }
                    }
                    
                }
                
                /*Ahora de entre el array de tareas encontradas en el fichero hay que encontrar la referencias de los objetos
                 creados e introducirlos al arraylist de la dependencia*/
                
                for (int i = 0;i<tareas.size();i++) {
                    Tarea aux = (Tarea) tareas.get(i);
                    
                    if (aux.nombre.compareTo(tarea1) == 0) {
                        task1 = aux;
                        continue;
                        
                    } else if (aux.nombre.compareTo(tarea2) == 0) {
                        task2 = aux;
                        continue;
                    }
                    
                    if (task1 != null && task2 != null)
                        break;
                    
                    
                }
                             
                if (task1 ==  null && task2 == null) throw new ErrorSintacticoException(0,"Tasks involved in a dependency do not exist in the current WorkFlow");
      
                dependencia = new Dependencia(nombre,task1,task2);
                
                for (int i = 0;i<ficheros.size();i++) {
                    dependencia.anadeFichero((Fichero)ficheros.get(i));
                }
                
                
                
            }
            
            else throw new ErrorSintacticoException(0,"Dependency name could not be found!");
        }
        
        else throw new ErrorSintacticoException(0,"Dependency tag expected but not found");
        
        return dependencia;
        
    }
    
    
    public WorkFlow leerWorkFlow()   {
        WorkFlow w = null;
        
        List list = this.document.getRootElement().getChildren();
        
        if (list.size() > 0) {
            Element workflow = (Element) list.get(0);
            
            if (workflow.getName().compareTo("WorkFlow") != 0)
                return null;
            
            String name = workflow.getAttribute("Name").getValue();
            
            
            //Si el atributo no existe creo un workflow con un nombre aleatorio, que comienza por
            //generic y termina con un número aleatorio para que el nombre no se repita.
            
            if (name != null && name.compareTo("") != 0)
                w = new WorkFlow(interfaz,name);
            
            else
                w = new WorkFlow(interfaz,"Generic"+Math.random());
            
            List list2 = workflow.getChildren();
            try {
                
                for (int i = 0;i< list2.size();i++) {
                    Element aux = (Element)list2.get(i);
                    
                    if (aux.getName().compareTo("Tasks") == 0) {
                        List listatareas = aux.getChildren();
                        
                        for (int j = 0;j<listatareas.size();j++) {
                            Element task = (Element)listatareas.get(j);
                            Tarea tarea = this.leerTarea(task);
                            if (w.contieneTarea(tarea) == true)
                                throw new ErrorSintacticoException(0,"Error, There are tasks with the same name. Corrupted file!");
                            w.crearTarea(tarea,tarea.x_pos,tarea.y_pos);
                        }
                    }
                    
                    else if (aux.getName().compareTo("Dependencies") == 0 ) {
                        List listadependencies = aux.getChildren();
                        
                        for (int j = 0;j<listadependencies.size();j++) {
                            Element dependency = (Element)listadependencies.get(j);
                            Dependencia nuevadep = this.leerDependencia(dependency,w.listaTareas);
                            w.listaDeps.add(nuevadep);
                        }
                    }
                    
                    else throw new ErrorSintacticoException(0,"Error, unexpected tag found!");
                }
                
                
                
            }catch(Exception e) {
                JOptionPane.showMessageDialog(interfaz, e.toString(), "Error", JOptionPane.ERROR_MESSAGE);
                return null;
            }
        }
        
   
        
        for (int i = 0;i<w.listaDeps.size();i++) {
            Dependencia aux = (Dependencia) w.listaDeps.get(i);
            
            aux.nombre = "Dep"+i;
            
            DefaultEdge dep = (DefaultEdge)w.createEdge(aux.nombre,w.getDefaultGraphCellFromTaskName(aux.tarea1.nombre),
                    w.getDefaultGraphCellFromTaskName(aux.tarea2.nombre));
            
            w.graph.getGraphLayoutCache().insert(dep);
            
            
        }
        
        w.depcounter = w.listaDeps.size(); //Inicializa los contadores
        
        w.ruta = this.file.getPath();
        
        return w;
        
    }
    
    public void Cargar() throws JDOMException, IOException {
        /*Carga el fichero y construye el documento xml, a partir de documentos obtendremos la información necesaria
         para reconstruir el workflow con todas sus tareas y dependencias.*/
        
        SAXBuilder builder=new SAXBuilder(false);
        document = builder.build(file);
    }
    
    public void Guardar() throws IOException {
        if (file.exists() == true) {
            JOptionPane dialog = new JOptionPane();
            String string1 = "Yes";
            String string2 = "No";
            Object[] options = {string1, string2};
            
            
            int n = JOptionPane.showOptionDialog(null,
                    "The file already exists, would you like to continue?",
                    "Caution",
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    null,     //don't use a custom Icon
                    options,  //the titles of buttons
                    string2); //the title of the default button
            
            
            if (n == 1)
                return;
            
            
        }
        
        XMLOutputter outputter = new XMLOutputter();
        FileWriter writer = new FileWriter(file);
        outputter.output(document,writer);
        writer.close();
    }
    
    
    
    private void crearDocumento() {
        Element root = new Element("WorkFlows");
        document = new Document(root);
    }
    
    public void anadeWorkFlow(WorkFlow w) {
        
        Element workflow = new Element("WorkFlow");
        
        workflow.setAttribute("Name",w.name);
        
        Element Tasks = new Element("Tasks");
        
        
        ArrayList listaTareas = w.listaTareas;
        
        /*********Se almacena la información de las tareas********************/
        for (int i = 0;i<listaTareas.size();i++) {
            Tarea aux = (Tarea)listaTareas.get(i);
            this.anadeTarea(Tasks,aux,w);
        }
        
        /*********Se almacena la información de las dependencias*************/
        
        ArrayList listaDeps = w.listaDeps;
        
        Element Dependencies = new Element("Dependencies");
        
        
        for (int i = 0;i<listaDeps.size();i++) {
            Dependencia aux = (Dependencia)listaDeps.get(i);
            this.anadeDependencia(Dependencies,aux);
        }
        
        workflow.getChildren().add(Tasks);
        
        workflow.getChildren().add(Dependencies);
        
        
        document.getRootElement().addContent(workflow);
        
        
    }
    
    private void anadeDependencia(Element workflow,Dependencia d) {
        Element dependency = new Element("Dependency");
        
        dependency.setAttribute("Name",d.nombre);
        
        Element Task1 = new Element("Task1").addContent(d.tarea1.nombre);
        
        Element Task2 = new Element("Task2").addContent(d.tarea2.nombre);
        
        ArrayList ficherosinvolucrados = d.obtieneFicheros();
        
        Element Files = new Element("Files");
        
        
        
        for (int i = 0;i<ficherosinvolucrados.size();i++) {
            Fichero aux = (Fichero)ficherosinvolucrados.get(i);
            Element file = new Element("File").addContent(aux.toString());
            
            Files.getChildren().add(file);
        }
        
        dependency.getChildren().add(Task1);
        
        dependency.getChildren().add(Task2);
        
        dependency.getChildren().add(Files);
        
        workflow.getChildren().add(dependency);
    }
    
    
    private void anadeNumeroTareas(Element tarea, int numeroTareas)
    {
        
        Element bulk = new Element("BulkJobInfo");
        
        bulk.setAttribute("number_of_tasks",""+numeroTareas);
        
        tarea.getChildren().add(bulk);
    }
    
    
    private void anadeTarea(Element workflow, Tarea t,WorkFlow w) {
        
        Element tarea = new Element("Task");
        
        tarea.setAttribute("Name",t.nombre);
        
        
        if (t instanceof TareaMultiple == true)
            this.anadeNumeroTareas(tarea,((TareaMultiple)t).numeroTareas);
        
        /*******Añade el ejecutable*******************************************/
        this.anadeEjecutable(tarea,t.ejecutable.toString(),t.ejecutable.fichero_local);
        
        /*********Crea un elemento argumentos*****************/
        
        this.anadeArgumentos(tarea,t.argumentos);
        
        
        /*********Añade las variables de entorno******************************/
        this.anadeVariables(tarea,t.variables);
        
        
        /************Añade tipo********************************************/
        
        this.anadeTipo(tarea,t.tipo,t.np);
        
        /***********Añade la información de planificación*************************/
        this.anadeSchedulingInformation(tarea,t.intervaloPlanificacion,t.umbralPlanificacion,t.tiempoDeadline);
        
        
        /*******************Añade los ficheros de entrada***************************/
        
        this.anadeFicherosEntrada(tarea,t.ficherosEntrada);
        
        /******************Añade los ficheros de salida*****************************/
        
        this.anadeFicherosSalida(tarea,t.ficherosSalida);
        
        /******************Añade los streams stdin, stdout, stderr******************/
        
        this.anadeStreams(tarea,t.stdin.toString(),t.stdout.toString(),t.stderr.toString(),t.stdin.fichero_local,
                t.stdout.fichero_local,
                t.stderr.fichero_local);
        
        /*****************Añade información de checkpoint*************************/
        
        this.anadeCheckPointing(tarea,t.restartFile.toString(),t.intervaloCheckpoint,t.checkpointURL);
        
        /****************Añade requisitos de tarea**********************************/
        
        this.anadeRequisitos(tarea,t.requisitosTarea);
        
        /***************Añade ranking de tarea******************************************/
        
        this.anadeRanking(tarea,t.rankingTarea);
        
        
        /***************Añade información del rendimiento***********************************/
        if (t.programa_monitor != null)
            this.anadePerformanceInformation(tarea,t.tiempo_maximo_de_suspension,t.umbral_carga_cpu,t.programa_monitor.toString(),
                    t.programa_monitor.fichero_local);
        
        else
            this.anadePerformanceInformation(tarea,t.tiempo_maximo_de_suspension,
                    t.umbral_carga_cpu,null,false);
        
        /**************Información de tolerancia de fallos*********************************/
        
        this.anadeFaultToleranceInformation(tarea,t.planificarSiFallo,t.numeroReintentos);
        
        /**************Añade información avanzada***************************************/
        
        this.anadeAdvanceInformation(tarea,t.wrapper.toString(),t.preWrapper.toString(),t.argumentosPreWrapper);
        
        Rectangle2D rect = w.getTaskPosition(t.nombre);
        
        if (rect != null) //Si falla es raro
            this.anadePosicionTarea(tarea,rect);
        
        workflow.getChildren().add(tarea);
    }
    
    
    private void anadePosicionTarea(Element tarea, Rectangle2D rect) {
        Element POSITION = new Element("POSITION");
        POSITION.setAttribute("x",""+rect.getX());
        POSITION.setAttribute("y",""+rect.getY());
        
        tarea.getChildren().add(POSITION);
    }
    
    
    private void anadeAdvanceInformation(Element tarea, String Wrapper, String preWrapper, String preWrapperArguments) {
        Element WRAPPER = new Element("Wrapper").addContent(Wrapper);
        
        Element PREWRAPPER = new Element("PreWrapper").addContent(preWrapper);
        
        Element PREWRAPPERARGUMENTS = new Element("PreWrapperArguments");
        
        this.anadeArgumentos(PREWRAPPERARGUMENTS,preWrapperArguments); //Es igual para los argumentos de tarea que para los del prewrapper
        
        if (Wrapper.compareTo("") != 0)
            tarea.getChildren().add(WRAPPER);
        
        if (preWrapper.compareTo("") != 0)
            tarea.getChildren().add(PREWRAPPER);
        
        if (preWrapperArguments.compareTo("") != 0)
            tarea.getChildren().add(PREWRAPPERARGUMENTS);
    }
    
    private void anadeFaultToleranceInformation(Element tarea, boolean replanificar, int reintentos) {
        Element tolerancia= new Element("FAULT_TOLERANCE");
        
        if (replanificar == true)
            tolerancia.setAttribute("Reschedule_on_failure","yes");
        else
            tolerancia.setAttribute("Reschedule_on_failure","no");
        
        tolerancia.setAttribute("Retries",""+reintentos);
        
        tarea.getChildren().add(tolerancia);
        
    }
    
    private void anadePerformanceInformation(Element tarea,int suspension_timeout,int cpuloadThreshold,String Monitor,boolean local) {
        Element Performance = new Element("PERFORMANCE_INFORMATION");
        
        Element Suspension_timeout = new Element("SUSPENSION_TIMEOUT").addContent(""+suspension_timeout);
        
        Element CPUloadThreshold = new Element("CPULOAD_THRESHOLD").addContent(""+cpuloadThreshold);
        
        
        Element MONITOR = new Element("MONITOR");
        if (Monitor != null && Monitor.compareTo("") != 0) {
            MONITOR.addContent(Monitor);
            
            if (local == true)
                MONITOR.setAttribute("local","yes");
            else
                MONITOR.setAttribute("local","no");
        }
        
        Performance.getChildren().add(Suspension_timeout);
        Performance.getChildren().add(CPUloadThreshold);
        
        Performance.getChildren().add(MONITOR);
        
        tarea.getChildren().add(Performance);
    }
    
    
    
    private void anadeRanking(Element tarea, String ranking) {
        Element RANKING = new Element("RANKING");
        
        RANKING.addContent(ranking);
        
        tarea.getChildren().add(RANKING);
    }
    
    private void anadeRequisitos(Element tarea, String requirements) {
        Element REQUIREMENTS = new Element("REQUIREMENTS");
        REQUIREMENTS.addContent(requirements);
        
        tarea.getChildren().add(REQUIREMENTS);
    }
    
    
    
    private void anadeCheckPointing(Element tarea,String restartFile, int interval, String URL) {
        Element Checkpoint = new Element("CHECKPOINTING");
        
        Element RESTART_FILES = new  Element("RESTART_FILES");
        
        Element Interval = new Element("Interval").addContent(""+interval);
        
        Element File = new Element("FILE").addContent(restartFile).setAttribute("local","no");
        
        Element URLCheckpoint = new Element("CHECKPOINT_URL").addContent(URL);
        
        RESTART_FILES.getChildren().add(File);
        
        if (restartFile != null && restartFile.compareTo("") != 0)
            Checkpoint.getChildren().add(RESTART_FILES);
        
        if (restartFile != null && restartFile.compareTo("") != 0)
            Checkpoint.getChildren().add(URLCheckpoint);
        
        Checkpoint.getChildren().add(Interval);
        
        tarea.getChildren().add(Checkpoint);
        
    }
    
    private void anadeStreams(Element tarea,String stdin, String stdout, String stderr,boolean localstdin,
            boolean localstdout,
            boolean localstderr) {
        Element Streams = new Element("Streams");
        
        
        Element STDIN  = new Element("STDIN").addContent(stdin);
        
        if (localstdin == true)
            STDIN.setAttribute("local","yes");
        else
            STDIN.setAttribute("local","no");
        
        
        Element STDOUT = new Element("STDOUT").addContent(stdout);
        
        if (localstdout == true)
            STDOUT.setAttribute("local","yes");
        else
            STDOUT.setAttribute("local","no");
        
        
        
        Element STDERR = new Element("STDERR").addContent(stderr);
        
        if (localstderr == true)
            STDERR.setAttribute("local","yes");
        else
            STDERR.setAttribute("local","no");
        
        if (stdin.compareTo("") != 0)
            Streams.getChildren().add(STDIN);
        
        if (stdout.compareTo("") != 0)
            Streams.getChildren().add(STDOUT);
        
        if (stderr.compareTo("") != 0)
            Streams.getChildren().add(STDERR);
        
        tarea.getChildren().add(Streams);
    }
    
    private void anadeFicherosSalida(Element tarea,ArrayList ficherosSalida) {
        Element OutPutFiles = new Element("OUTPUT_FILES");
        
        for (int i = 0;i<ficherosSalida.size();i++) {
            Element file = new Element("FILE").addContent(ficherosSalida.get(i).toString());
            
            Fichero out = (Fichero)ficherosSalida.get(i);
            
            if (out.fichero_local == true)
                file.setAttribute("local","yes");
            else
                file.setAttribute("local","no");
            
            OutPutFiles.getChildren().add(file);
        }
        
        tarea.getChildren().add(OutPutFiles);
    }
    
    private void anadeFicherosEntrada(Element tarea,ArrayList ficherosEntrada) {
        Element InPutFiles = new Element("INPUT_FILES");
        
        for (int i = 0;i<ficherosEntrada.size();i++) {
            
            Element file = new Element("FILE").addContent(ficherosEntrada.get(i).toString());
            Fichero in = (Fichero)ficherosEntrada.get(i);
            if (in.fichero_local == true)
                file.setAttribute("local","yes");
            else
                file.setAttribute("local","no");
            
            InPutFiles.getChildren().add(file);
        }
        
        tarea.getChildren().add(InPutFiles);
    }
    
    private void anadeSchedulingInformation(Element tarea, int intervalo, int umbralPlanificacion,Date tiempoDeadline) {
        Element Scheduling = new Element("Scheduling");
        
        Element Interval = new Element("Interval").addContent(""+intervalo);
        
        Element Reschedule_threshold = new Element("Reschedule_threshold").addContent(""+umbralPlanificacion);
        
        ///////////////////////////////////////////////////////////////////////
        Element deadline = new Element("Deadline");
        if (tiempoDeadline != null) {
            deadline.addContent(
                    tiempoDeadline.getTime()+"");
        }
        ///////////////////////////////////////////////////////////////////////
        
        Scheduling.getChildren().add(Interval);
        Scheduling.getChildren().add(Reschedule_threshold);
        Scheduling.getChildren().add(deadline);
        
        tarea.getChildren().add(Scheduling);
        
    }
    
    
    private void anadeTipo(Element tarea, int tipo,int NP) {
        Element Type = new Element("Type");
        
        switch(tipo) {
            case 0:
                Type.addContent("SINGLE");
                break;
            case 1:
                Type.addContent("MULTIPLE");
            case 2:
                Type.addContent("MPI");
                
            default:
                Type.addContent("SINGLE");
                break;
        }
        
        Type.setAttribute("NP",""+NP);
        
        tarea.getChildren().add(Type);
        
        
    }
    
    private void anadeEjecutable(Element tarea,String ejecutable, boolean local) {
        Element Ejecutable = new Element("Executable");
        if (local == true)
            Ejecutable.setAttribute("local","yes");
        else
            Ejecutable.setAttribute("local","no");
        
        
        if (ejecutable.replace(" ","").compareTo("") == 0)
            return;
        
        Ejecutable.addContent(ejecutable);
        
        tarea.getChildren().add(Ejecutable);
        
    }
    
    
    private void anadeArgumentos(Element tarea,String argumentos) {
        
        String[] args = argumentos.split(" ");
        
        Element Argumentos = new Element("Arguments");
        
        if (args.length == 1 && args[0].compareTo("") == 0) {
            tarea.getChildren().add(Argumentos);
            return;
        }
        
        for (int i = 0;i<args.length;i++) {
            Element Argumento = new Element("Argument").addContent(args[i]);
            Argumentos.getChildren().add(Argumento);
        }
        
        tarea.getChildren().add(Argumentos);
        
        
    }
    
    private void anadeVariables(Element tarea,String variables) {
        String[] vars = variables.split(",");
        Element Variables = new Element("Environment");
        
        
        if (vars.length == 1 && vars[0].compareTo("") == 0) {
            tarea.getChildren().add(Variables);
            return;
        }
        
        
        
        for (int i = 0;i<vars.length;i++) {
            
            Element variable = new Element("Variable").addContent(vars[i]);
            Variables.getChildren().add(variable);
            
        }
        
        tarea.getChildren().add(Variables);
    }
}
